#ifndef INCLUDE_kyut_EmbeddingMethod_hpp
#define INCLUDE_kyut_EmbeddingMethod_hpp

#include <string_view>
#include <unordered_map>
#include <boost/optional.hpp>

namespace kyut {
    enum class EmbeddingMethod {
        FunctionOrdering,
        OperandSwapping,
    };

    constexpr std::string_view to_string(EmbeddingMethod m) noexcept {
        switch (m) {
            case EmbeddingMethod::FunctionOrdering:
                return "function-ordering";
            case EmbeddingMethod::OperandSwapping:
                return "operand-swapping";
            default:
                return "unknown";
        }
    }

    inline boost::optional<EmbeddingMethod> embedding_method_from(std::string_view s) noexcept {
        static const std::unordered_map<std::string_view, EmbeddingMethod> embedding_methods = {
            {to_string(EmbeddingMethod::FunctionOrdering), EmbeddingMethod::FunctionOrdering},
            {to_string(EmbeddingMethod::OperandSwapping), EmbeddingMethod::OperandSwapping},
        };

        if (const auto it = embedding_methods.find(s); it != std::end(embedding_methods)) {
            return it->second;
        }
        return boost::none;
    }

    inline std::ostream& operator<<(std::ostream& ostream, EmbeddingMethod m) {
        return ostream << to_string(m);
    }
} // namespace kyut

#endif // INCLUDE_kyut_EmbeddingMethod_hpp
